@@ -31,10 +31,10 @@ db3 = canmatrix.CanMatrix()
 #
 
 # Copy Can-ID 1234 from second CAN-Matrix to target-Matrix
-canmatrix.copy.copy_frame(1234, db2, db3)
+canmatrix.copy.copy_frame(canmatrix.ArbitrationId(100), db2, db3)
 
 # Copy frame "Engine_123" from first CAN-Matrix to target-Matrix
-canmatrix.copy.copy_frame("Engine_123", db1, db3)
+canmatrix.copy.copy_frame(canmatrix.ArbitrationId(200), db1, db3)
 
 # Copy ECU (with all Frames) "Gateway" from first CAN-Matrix to target-Matrix
 canmatrix.copy.copy_ecu_with_frames("Gateway", db1, db3)
@@ -49,4 +49,4 @@ canmatrix.copy.copy_ecu_with_frames("Gateway", db1, db3)
 # export the new (target)-Matrix for example as .dbc:
 #
 
-canmatrix.formats.dumpp(db3, "target.dbc")
+canmatrix.formats.dumpp({"":db3}, "target.dbc")
