@@ -39,8 +39,8 @@ import warnings
 from builtins import *
 
 import attr
-from future.moves.itertools import zip_longest
 from past.builtins import basestring
+from six.moves import zip_longest
 
 import canmatrix.copy
 import canmatrix.types
@@ -51,10 +51,12 @@ if attr.__version__ < '17.4.0':  # type: ignore
 logger = logging.getLogger(__name__)
 defaultFloatFactory = decimal.Decimal  # type: typing.Callable[[typing.Any], canmatrix.types.PhysicalValue]
 
+
 class ExceptionTemplate(Exception):
     def __call__(self, *args):
         return self.__class__(*(self.args + args))
 
+
 class StartbitLowerZero(ExceptionTemplate): pass
 class EncodingComplexMultiplexed(ExceptionTemplate): pass
 class MissingMuxSignal(ExceptionTemplate): pass
@@ -62,6 +64,8 @@ class DecodingComplexMultiplexed(ExceptionTemplate): pass
 class DecodingFrameLength(ExceptionTemplate): pass
 class ArbitrationIdOutOfRange(ExceptionTemplate): pass
 class J1939needsExtendedIdetifier(ExceptionTemplate): pass
+class DecodingConatainerPdu(ExceptionTemplate): pass
+class EncodingConatainerPdu(ExceptionTemplate): pass
 
 
 def arbitration_id_converter(source):  # type: (typing.Union[int, ArbitrationId]) -> ArbitrationId
@@ -121,7 +125,7 @@ def normalize_value_table(table):  # type: (typing.Mapping) -> typing.MutableMap
     return {int(k): v for k, v in table.items()}
 
 
-@attr.s(cmp=False)
+@attr.s(eq=False)
 class Signal(object):
     """
     Represents a Signal in CAN Matrix.
@@ -296,7 +300,10 @@ class Signal(object):
         :param int or str value: signal value (0xFF)
         :param str valueName: Human readable value description ("Init")
         """
-        self.values[int(str(value), 0)] = valueName
+        if isinstance(value, defaultFloatFactory):
+            self.values[value.to_integral()] = valueName
+        else:
+            self.values[int(str(value), 0)] = valueName
 
     def set_startbit(self, start_bit, bitNumbering=None, startLittle=None):
         """
@@ -319,9 +326,9 @@ class Signal(object):
         if startLittle is True and self.is_little_endian is False:
             start_bit = start_bit + 1 - self.size
         if start_bit < 0:
-            print("wrong start_bit found Signal: %s Startbit: %d" %
-                  (self.name, start_bit))
-            raise StartbitLowerZero
+            raise StartbitLowerZero(
+                "wrong start_bit found Signal: %s Startbit: %d" % (self.name, start_bit)
+            )
         self.start_bit = start_bit
 
     def get_startbit(self, bit_numbering=None, start_little=None):
@@ -394,7 +401,6 @@ class Signal(object):
 
         return self.offset + (self.float_factory(rawMax) * self.factor)
 
-
     def phys2raw(self, value=None):
         # type: (canmatrix.types.OptionalPhysicalValue) -> canmatrix.types.RawValue
         """Return the raw value (= as is on CAN).
@@ -405,6 +411,8 @@ class Signal(object):
         """
         if value is None:
             value = self.initial_value
+            if not (self.min <= value <= self.max):
+                value = self.min
 
         if isinstance(value, basestring):
             for value_key, value_string in self.values.items():
@@ -449,7 +457,7 @@ class Signal(object):
         return self.name
 
 
-@attr.s(cmp=False)
+@attr.s(eq=False)
 class SignalGroup(object):
     """
     Represents signal-group, containing multiple Signals.
@@ -457,6 +465,7 @@ class SignalGroup(object):
     name = attr.ib()  # type: str
     id = attr.ib()  # type: int
     signals = attr.ib(factory=list, repr=False)  # type: typing.MutableSequence[Signal]
+    e2e_trans = attr.ib(default=None)
 
     def add_signal(self, signal):  # type: (Signal) -> None
         """Add a Signal to SignalGroup.
@@ -535,6 +544,7 @@ def grouper(iterable, n, fillvalue=None):
     args = [iter(iterable)] * n
     return zip_longest(*args, fillvalue=fillvalue)
 
+
 def unpack_bitstring(length, is_float, is_signed, bits):
     # type: (int, bool, bool, typing.Any) -> typing.Union[float, int]
     """
@@ -562,6 +572,7 @@ def unpack_bitstring(length, is_float, is_signed, bits):
 
     return value
 
+
 def pack_bitstring(length, is_float, value, signed):
     """
     returns a value in bits
@@ -581,7 +592,7 @@ def pack_bitstring(length, is_float, value, signed):
         x = bytearray(struct.pack(float_type, value))
         bitstring = ''.join('{:08b}'.format(b) for b in x)
     else:
-        b = '{:0{}b}'.format(int((2<<length )+ value), length)
+        b = '{:0{}b}'.format(int((2 << length) + value), length)
         bitstring = b[-length:]
 
     return bitstring
@@ -746,7 +757,8 @@ class ArbitrationId(object):
             )
         )
 
-@attr.s(cmp=False)
+
+@attr.s(eq=False)
 class Pdu(object):
     """
     Represents a PDU.
@@ -759,11 +771,13 @@ class Pdu(object):
 
     name = attr.ib(default="")  # type: str
     size = attr.ib(default=0)  # type: int
+    id = attr.ib(default=0)  # type: int
     triggering_name = attr.ib(default="")  # type: str
     pdu_type = attr.ib(default="")  # type: str
     port_type = attr.ib(default="")  # type: str
     signals = attr.ib(factory=list)  # type: typing.MutableSequence[Signal]
     signalGroups = attr.ib(factory=list)  # type: typing.MutableSequence[SignalGroup]
+    cycle_time = attr.ib(default=0)  # type: int
 
     def add_signal(self, signal):
         # type: (Signal) -> Signal
@@ -775,7 +789,7 @@ class Pdu(object):
         """
         self.signals.append(signal)
         return self.signals[len(self.signals) - 1]
-    def add_signal_group(self, Name, Id, signalNames):
+    def add_signal_group(self, Name, Id, signalNames, e2e_trans=None):
         # type: (str, int, typing.Sequence[str]) -> None
         """Add new SignalGroup to the Frame. Add given signals to the group.
 
@@ -783,7 +797,7 @@ class Pdu(object):
         :param int Id: Group id
         :param list of str signalNames: list of Signal names to add. Non existing names are ignored.
         """
-        newGroup = SignalGroup(Name, Id)
+        newGroup = SignalGroup(Name, Id, e2e_trans=e2e_trans)
         self.signalGroups.append(newGroup)
         for signal in signalNames:
             signal = signal.strip()
@@ -814,7 +828,7 @@ class Pdu(object):
         return None
 
 
-@attr.s(cmp=False)
+@attr.s(eq=False)
 class Frame(object):
     """
     Represents CAN Frame.
@@ -860,7 +874,6 @@ class Frame(object):
     header_id = attr.ib(default=None)  #type: int
     # header_id
 
-
     @property
     def is_multiplexed(self):  # type: () -> bool
         """Frame is multiplexed if at least one of its signals is a multiplexer."""
@@ -900,7 +913,13 @@ class Frame(object):
                 muxed_signals.append(sig)
         return muxed_signals
 
+    @property
+    def is_pdu_container(self):  # type: () -> bool
+        return len(self.pdus) > 0
 
+    @property
+    def get_pdu_id_values(self):  # type: () -> typing.Sequence[int]
+        return list({pdu.id for pdu in self.pdus})
 
     @property
     def pgn(self):  # type: () -> int
@@ -930,7 +949,6 @@ class Frame(object):
         """Set J1939 source."""
         self.arbitration_id.j1939_source = value
 
-
     @property
     def effective_cycle_time(self):
         """Calculate effective cycle time for frame, depending on singal cycle times"""
@@ -940,8 +958,8 @@ class Frame(object):
         elif len(min_cycle_time_list) == 1:
             return min_cycle_time_list[0]
         else:
-            gcd = canmatrix.utils.get_gcd(min_cycle_time_list[0],min_cycle_time_list[1])
-            for i in range(2,len(min_cycle_time_list)):
+            gcd = canmatrix.utils.get_gcd(min_cycle_time_list[0], min_cycle_time_list[1])
+            for i in range(2, len(min_cycle_time_list)):
                 gcd = canmatrix.utils.get_gcd(gcd, min_cycle_time_list[i])
             return gcd
         #    return min(min_cycle_time_list)
@@ -979,7 +997,7 @@ class Frame(object):
 
         return iter(self.signals)
 
-    def add_signal_group(self, Name, Id, signalNames):
+    def add_signal_group(self, Name, Id, signalNames, e2e_trans=None):
         # type: (str, int, typing.Sequence[str]) -> None
         """Add new SignalGroup to the Frame. Add given signals to the group.
 
@@ -987,7 +1005,7 @@ class Frame(object):
         :param int Id: Group id
         :param list of str signalNames: list of Signal names to add. Non existing names are ignored.
         """
-        newGroup = SignalGroup(Name, Id)
+        newGroup = SignalGroup(Name, Id, e2e_trans=e2e_trans)
         self.signalGroups.append(newGroup)
         for signal in signalNames:
             signal = signal.strip()
@@ -1021,6 +1039,31 @@ class Frame(object):
         self.pdus.append(pdu)
         return self.pdus[len(self.pdus) - 1]
 
+    def pdu_by_name(self, name):
+        # type: (str) -> typing.Union[Pdu, None]
+        """Get PDU.
+
+        :param str name: PDU name
+        :return: PDU by name or None if not found.
+        :rtype: Pdu
+        """
+        for pdu in self.pdus:
+            if pdu.name == name:
+                return pdu
+        return None
+
+    def pdu_by_id(self, pdu_id):
+        # type: (int) -> typing.Union[Pdu, None]
+        """Get PDU.
+
+        :param int pdu_id: PDU id
+        :return: PDU by id or None if not found.
+        :rtype: Pdu
+        """
+        for pdu in self.pdus:
+            if pdu.id == pdu_id:
+                return pdu
+        return None
 
     def add_signal(self, signal):
         # type: (Signal) -> Signal
@@ -1131,7 +1174,11 @@ class Frame(object):
         for sig in self.signals:
             if sig.get_startbit() + int(sig.size) > max_bit:
                 max_bit = sig.get_startbit() + int(sig.size)
-        max_byte = int(math.ceil(max_bit / 8))
+        max_byte = (max_bit + 7) // 8
+        if self.is_pdu_container:
+            max_byte *= len(self.pdus)
+            for pdu in self.pdus:
+                max_byte += pdu.size
         self.size = max(self.size, max_byte)
 
     def fit_dlc(self):
@@ -1261,6 +1308,8 @@ class Frame(object):
 
         if self.is_complex_multiplexed:
             raise EncodingComplexMultiplexed
+        elif self.is_pdu_container:
+            raise EncodingConatainerPdu  # TODO add encoding
         elif self.is_multiplexed:
             # search for mulitplexer-signal
             for signal in self.signals:
@@ -1283,7 +1332,8 @@ class Frame(object):
             data = newData
         return self.signals_to_bytes(data)
 
-    def bytes_to_bitstrings(self, data):
+    @staticmethod
+    def bytes_to_bitstrings(data):
         # type: (bytes) -> typing.Tuple[str, str]
         """Return two arrays big and little containing bits of given data (bytearray)
 
@@ -1297,19 +1347,21 @@ class Frame(object):
 
         return little, big
 
-    def bitstring_to_signal_list(self, signals, big, little):
-        # type: (typing.Sequence[Signal], str, str) -> typing.Sequence[canmatrix.types.RawValue]
+    @staticmethod
+    def bitstring_to_signal_list(signals, big, little, size):
+        # type: (typing.Sequence[Signal], str, str, int) -> typing.Sequence[canmatrix.types.RawValue]
         """Return OrderedDictionary with Signal Name: object decodedSignal (flat / without support for multiplexed frames)
 
         :param signals: Iterable of signals (class signal) to decode from frame.
         :param big: bytearray of bits (big endian).
         :param little: bytearray of bits (little endian).
+        :param size: number of bits.
         :return: array with raw values (same order like signals)
         """
         unpacked = []
         for signal in signals:
             if signal.is_little_endian:
-                least = self.size * 8 - signal.start_bit
+                least = size - signal.start_bit
                 most = least - signal.size
 
                 bits = little[most:least]
@@ -1318,7 +1370,6 @@ class Frame(object):
                 least = most + signal.size
 
                 bits = big[most:least]
-
             unpacked.append(unpack_bitstring(signal.size, signal.is_float, signal.is_signed, bits))
 
         return unpacked
@@ -1336,20 +1387,73 @@ class Frame(object):
 
         rx_length = len(data)
         if rx_length != self.size and report_error:
-            print(
-                'Received message 0x{self.arbitration_id.id:08X} with length {rx_length}, expected {self.size}'.format(**locals()))
-            raise DecodingFrameLength
+            raise DecodingFrameLength(
+                'Received message 0x{self.arbitration_id.id:08X} '
+                'with length {rx_length}, expected {self.size}'.format(**locals())
+            )
+        elif self.is_pdu_container:
+            header_id_signal = self.signal_by_name("Header_ID")
+            header_dlc_signal = self.signal_by_name("Header_DLC")
+            if header_id_signal is None or header_dlc_signal is None:
+                raise DecodingConatainerPdu(
+                    'Received message 0x{:08X} without Header_ID or '
+                    'Header_DLC signal'.format(self.arbitration_id.id)
+                )
+            # TODO: may be we need to check that ID/DLC signals are contiguous
+            header_size = header_id_signal.size + header_dlc_signal.size
+            little, big = self.bytes_to_bitstrings(data)
+            size = self.size * 8
+            return_dict = dict({"pdus": []})
+            # decode signal which are not in PDUs
+            signals = [s for s in self.signals if s not in [header_id_signal, header_dlc_signal]]
+            if signals:
+                unpacked = self.bitstring_to_signal_list(signals, big, little, size)
+                for s, v in zip(signals, unpacked):
+                    return_dict[s.name] = DecodedSignal(v, s)
+            # decode PDUs
+            offset = header_id_signal.start_bit
+            header_signals = [header_id_signal, header_dlc_signal]
+            while (offset + header_size) < size:
+                unpacked = self.bitstring_to_signal_list(
+                    header_signals,
+                    big[offset:offset + header_size],
+                    little[size - offset - header_size:size - offset],
+                    header_size
+                )
+                offset += header_size
+                pdu_id = unpacked[0]
+                pdu_dlc = unpacked[1]
+                for s, v in zip(header_signals, unpacked):
+                    if s.name not in return_dict:
+                        return_dict[s.name] = []
+                    return_dict[s.name].append(DecodedSignal(v, s))
+                pdu = self.pdu_by_id(pdu_id)
+                if pdu is None:
+                    return_dict['pdus'].append(None)
+                else:
+                    unpacked = self.bitstring_to_signal_list(
+                        pdu.signals,
+                        big[offset:offset + pdu_dlc * 8],
+                        little[size - offset - pdu_dlc * 8:size - offset],
+                        pdu_dlc * 8
+                    )
+                    pdu_dict = dict()
+                    for s, v in zip(pdu.signals, unpacked):
+                        pdu_dict[s.name] = DecodedSignal(v, s)
+                    return_dict["pdus"].append({pdu.name: pdu_dict})
+                offset += (pdu_dlc * 8)
+            return return_dict
         else:
             little, big = self.bytes_to_bitstrings(data)
 
-            unpacked = self.bitstring_to_signal_list(self.signals, big, little)
+            unpacked = self.bitstring_to_signal_list(self.signals, big, little, self.size * 8)
 
-            returnDict= dict()
+            return_dict = dict()
 
             for s, v in zip(self.signals, unpacked):
-                returnDict[s.name] = DecodedSignal(v, s)
+                return_dict[s.name] = DecodedSignal(v, s)
 
-            return returnDict
+            return return_dict
 
     def _get_sub_multiplexer(self, parent_multiplexer_name, parent_multiplexer_value):
         """
@@ -1413,7 +1517,7 @@ class Frame(object):
                 decoded_values[signal.name] = decoded[signal.name]
             return decoded_values
 
-        elif self.is_multiplexed:
+        elif self.is_multiplexed and not self.is_pdu_container:
             decoded_values = dict()
             # find multiplexer and decode only its value:
 
@@ -1513,12 +1617,13 @@ class Define(object):
 
 import enum
 
+
 class matrix_class(enum.Enum):
     CAN = 1
     FLEXRAY = 2
     SOMEIP = 3
 
-@attr.s(cmp=False)
+@attr.s(eq=False)
 class CanMatrix(object):
     """
     The Can-Matrix-Object
@@ -1713,8 +1818,8 @@ class CanMatrix(object):
                 for signal in frame.signals:
                     if signal_define in signal.attributes:
                         break
-            else:
-                defines_to_delete.add(signal_define)
+                else:
+                    defines_to_delete.add(signal_define)
         for element in defines_to_delete:
             del self.signal_defines[element]
 
@@ -1865,7 +1970,12 @@ class CanMatrix(object):
                 for sig in frame.signals:
                     if sig.get_startbit() + int(sig.size) > maxBit:
                         maxBit = sig.get_startbit() + int(sig.size)
-                frame.size = math.ceil(maxBit / 8)
+                max_byte = (maxBit + 7) // 8
+                if frame.is_pdu_container:
+                    max_byte *= len(frame.pdus)
+                    for pdu in self.pdus:
+                        max_byte += pdu.size
+                frame.size = max_byte
 
     def rename_ecu(self, ecu_or_name, new_name):  # type: (typing.Union[Ecu, str], str) -> None
         """Rename ECU in the Matrix. Update references in all Frames.
@@ -2088,6 +2198,15 @@ class CanMatrix(object):
         """
         return self.frame_by_id(frame_id).encode(data)
 
+    def decode_pycan(self, pycan_msg):
+        """Return OrderedDictionary with Signal Name: object decodedSignal
+
+        :param pycan_msg: python-can message
+        :return: OrderedDictionary
+        """
+        canmatrix_arbitration_id = canmatrix.ArbitrationId(pycan_msg.arbitration_id, extended=pycan_msg.is_extended_id)
+        return self.decode(canmatrix_arbitration_id, pycan_msg.data)
+
     def decode(self, frame_id, data):  # type: (ArbitrationId, bytes) -> typing.Mapping[str, typing.Any]
         """Return OrderedDictionary with Signal Name: object decodedSignal
 
